diff mbox series

[iproute2-next] ss: use COLUMNS from the environment, if TIOCGWINSZ fails

Message ID 021ce0ff240d23c1feca857cfd2338e525d9345f.1716285674.git.dsimic@manjaro.org (mailing list archive)
State Accepted
Delegated to: David Ahern
Headers show
Series [iproute2-next] ss: use COLUMNS from the environment, if TIOCGWINSZ fails | expand

Checks

Context Check Description
netdev/tree_selection success Not a local patch

Commit Message

Dragan Simic May 21, 2024, 10:04 a.m. UTC
Use the COLUMNS environment variable [1] when determining the screen width,
if using TIOCGWINSZ isn't possible or if it fails.  This allows better use
of the available horizontal screen space in certain scenarios, and makes
the produced outputs more readable, as described further below.

All major shells can maintain the COLUMNS variable according to the current
screen size, [2][3][4] but this shell variable isn't actually an environment
variable, i.e. it doesn't get exported to the shell subprocesses by default.
For example, no COLUMNS environment variable reaches ss(8) when it's executed
as part of a shell pipeline or inside a shell script.

Though, users can opt to export the COLUMNS variable by hand, or they can
rely on some other utilities to do that for them.  A good example of such
utilities is watch(1) that exports COLUMNS as an environment variable to
the processes it executes. [5]  Using ss(8) together with watch(1) is rather
useful, and honoring the exported COLUMNS variable makes the outputs produced
by ss(8) in this scenario more readable.

The behavior of shells, which don't export the COLUMNS variable by default,
makes this change safe in the sense of not affecting the usual shell pipeline
workflows or various shell scripts that use ss(8).

[1] https://pubs.opengroup.org/onlinepubs/9699919799.2016edition/basedefs/V1_chap08.html
[2] https://man.archlinux.org/man/bash.1.en#COLUMNS
[3] https://man.archlinux.org/man/tcsh.1.en#Terminal_management_(+)
[4] https://man.archlinux.org/man/zshall.1.en#Configuration
[5] https://gitlab.com/procps-ng/procps/-/blob/master/NEWS?ref_type=heads#L623

Signed-off-by: Dragan Simic <dsimic@manjaro.org>
---
 misc/ss.c | 15 ++++++++++++++-
 1 file changed, 14 insertions(+), 1 deletion(-)
diff mbox series

Patch

diff --git a/misc/ss.c b/misc/ss.c
index 8ff6e1002060..54f36b39c6bd 100644
--- a/misc/ss.c
+++ b/misc/ss.c
@@ -1175,20 +1175,33 @@  static void buf_free_all(void)
 	buffer.chunks = 0;
 }
 
 /* Get current screen width, returns -1 if TIOCGWINSZ fails */
 static int render_screen_width(void)
 {
 	int width = -1;
 
 	if (isatty(STDOUT_FILENO)) {
 		struct winsize w;
 
 		if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &w) != -1) {
 			if (w.ws_col > 0)
 				width = w.ws_col;
 		}
 	}
 
+	if (width == -1) {
+		const char *p = getenv("COLUMNS");
+		int c;
+
+		if (p) {
+			c = atoi(p);
+			if (c > 0)
+				width = c;
+		}
+	}
+
 	return width;
 }
@@ -1175,20 +1175,33 @@  static void buf_free_all(void)
 }
 
-/* Get current screen width, returns -1 if TIOCGWINSZ fails */
+/* Get current screen width. Returns -1 if TIOCGWINSZ fails and there's
+ * no COLUMNS variable in the environment.
+ */
 static int render_screen_width(void)
 {